from bluer_objects.README.items import ImageItems
from bluer_objects.README.consts import designs_url

from bluer_ugv.README.rangin.consts import rangin_assets2

docs = [
    {
        "path": "../docs/rangin/flag.md",
        "cols": 2,
        "items": ImageItems(
            {
                f"{rangin_assets2}/flag/sketch.jpg": "",
                designs_url(
                    "rangin/flag/design.png?raw=true",
                ): designs_url(
                    "rangin/flag/design.svg",
                ),
            }
        ),
    },
]



import argparse

from blueness import module
from blueness.argparse.generic import sys_exit

from bluer_ugv import NAME
from bluer_ugv.swallow.targeting import select_target
from bluer_ugv.swallow.debug import debug
from bluer_ugv.logger import logger

NAME = module.name(__file__, NAME)

parser = argparse.ArgumentParser(NAME)
parser.add_argument(
    "task",
    type=str,
    help="debug|select_target",
)
parser.add_argument(
    "--host",
    type=str,
)
parser.add_argument(
    "--loop",
    type=int,
    default=1,
    help="0 | 1",
)
parser.add_argument(
    "--object_name",
    type=str,
)
parser.add_argument(
    "--save_images",
    type=int,
    default=1,
    help="0 | 1",
)
parser.add_argument(
    "--generate_gif",
    type=int,
    default=1,
    help="0 | 1",
)
args = parser.parse_args()

success = False
if args.task == "debug":
    success = debug(
        object_name=args.object_name,
        generate_gif=args.generate_gif == 1,
        save_images=args.save_images == 1,
    )
elif args.task == "select_target":
    success = select_target(
        host=args.host,
        loop=args.loop == 1,
    )
else:
    success = None

sys_exit(logger, NAME, args.task, success)



from bluer_objects.README.items import ImageItems
from bluer_sbc.parts.db import db_of_parts
from bluer_sbc.parts.consts import parts_url_prefix

from bluer_ugv.README.arzhang.consts import arzhang_mechanical_design
from bluer_ugv.designs.arzhang.parts import dict_of_parts

docs = [
    {
        "path": "../docs/arzhang/design",
    },
    {
        "path": "../docs/arzhang/design/specs.md",
    },
    {
        "path": "../docs/arzhang/design/parts.md",
        "items": db_of_parts.as_images(
            dict_of_parts,
            reference=parts_url_prefix,
        ),
        "macros": {
            "parts:::": db_of_parts.as_list(
                dict_of_parts,
                reference=parts_url_prefix,
                log=False,
            ),
        },
    },
    {
        "path": "../docs/arzhang/design/mechanical",
        "cols": 2,
        "items": ImageItems(
            {
                f"{arzhang_mechanical_design}/robot-with-cover-v2.png": f"{arzhang_mechanical_design}/robot.stl",
                f"{arzhang_mechanical_design}/robot.png": f"{arzhang_mechanical_design}/robot.stl",
                f"{arzhang_mechanical_design}/cage.png": f"{arzhang_mechanical_design}/cage.stl",
                f"{arzhang_mechanical_design}/measurements.png": "",
            }
        ),
    },
    {
        "path": "../docs/arzhang/design/mechanical/v1.md",
        "items": ImageItems(
            {
                f"{arzhang_mechanical_design}/v1/robot.png": f"{arzhang_mechanical_design}/v1/robot.stl",
                f"{arzhang_mechanical_design}/v1/cage.png": f"{arzhang_mechanical_design}/v1/cage.stl",
                f"{arzhang_mechanical_design}/v1/measurements.png": "",
            }
        ),
    },
]



from typing import List, Tuple

from bluer_ugv import env
from bluer_ugv.swallow.session.classical.ultrasonic_sensor.sensor import (
    lUltrasonicSensor,
)
from bluer_ugv.swallow.session.classical.ultrasonic_sensor.detection import Detection
from bluer_ugv.logger import logger


class UltrasonicSensorPack:
    def __init__(
        self,
        setmode: bool = True,
        max_m: float = env.BLUER_UGV_ULTRASONIC_SENSOR_MAX_M,
    ) -> None:
        self.left = lUltrasonicSensor(
            side="left",
            setmode=setmode,
            max_m=max_m,
        )
        self.right = lUltrasonicSensor(
            side="right",
            setmode=False,
            max_m=max_m,
        )

        self.valid = self.left.valid and self.right.valid

    def detect(
        self,
        log: bool = True,
    ) -> Tuple[bool, List[Detection]]:
        success_left, detection_left = self.left.detect(log=False)
        success_right, detection_right = self.right.detect(log=False)

        if log:
            logger.info(
                " | ".join(
                    [
                        detection_left.as_str(),
                        detection_right.as_str(),
                    ]
                )
            )

        return (
            success_left and success_right,
            [detection_left, detection_right],
        )



from RPi import GPIO
from typing import Tuple
import time

from bluer_ugv import env
from bluer_ugv.logger import logger
from bluer_ugv.swallow.session.classical.ultrasonic_sensor.consts import (
    C,
    CYCLE_MIN_S,
    WAIT_LOW_TIMEOUT_S,
    WAIT_HIGH_TIMEOUT_S,
    TRIG_PULSE_S,
)
from bluer_ugv.swallow.session.classical.ultrasonic_sensor.detection import Detection


def monotonic_s():
    return time.monotonic_ns() * 1e-9


class lUltrasonicSensor:
    def __init__(
        self,
        side: str,
        setmode: bool = True,
        max_m: float = env.BLUER_UGV_ULTRASONIC_SENSOR_MAX_M,
    ):
        self.side = side
        self.valid = True

        self.max_m = max_m
        self.THRESH_S = (2 * max_m) / C  # round-trip time threshold

        # Pin definitions
        if side == "left":
            self.TRIG = 23  # GPIO 23, pin 16
            self.ECHO = 24  # GPIO 24, pin 18
        elif side == "right":  # right
            self.TRIG = 5  # GPIO 5,  pin 29
            self.ECHO = 25  # GPIO 25, pin 22
        else:
            logger.error(f"{side}: ultrasonic sensor not found.")
            self.valid = False

        if not self.valid:
            return

        if setmode:
            GPIO.setmode(GPIO.BCM)

        GPIO.setup(self.TRIG, GPIO.OUT, initial=GPIO.LOW)
        GPIO.setup(self.ECHO, GPIO.IN, pull_up_down=GPIO.PUD_DOWN)

        logger.info(
            "{}: {} ultrasonic sensor initialized on TRIG=GPIO#{}, ECHO=GPIO#{} at max distance={:6.2f} mm == {:6.2} ms.".format(
                self.__class__.__name__,
                self.side,
                self.TRIG,
                self.ECHO,
                self.max_m * 1000,
                self.THRESH_S * 1000,
            )
        )

    def detect(
        self,
        log: bool = True,
    ) -> Tuple[bool, Detection]:
        if not self.valid:
            logger.error("invalid ultrasonic sensor")
            return False, Detection(
                side=self.side,
                reason="invalid sensor",
            )

        detection = Detection(side=self.side)

        cycle_start = monotonic_s()

        # Trigger pulse
        GPIO.output(self.TRIG, GPIO.LOW)
        time.sleep(200e-6)  # settle
        GPIO.output(self.TRIG, GPIO.HIGH)
        time.sleep(TRIG_PULSE_S)  # 30 µs
        GPIO.output(self.TRIG, GPIO.LOW)

        # Wait for rising edge
        t0 = monotonic_s()
        while GPIO.input(self.ECHO) == 0 and (monotonic_s() - t0) < WAIT_HIGH_TIMEOUT_S:
            pass
        if GPIO.input(self.ECHO) == 0:
            detection = Detection(
                side=self.side,
                detection=False,
                reason="no echo high",
            )
        else:
            t_rise = monotonic_s()

            # Wait for falling edge
            t_fall_deadline = t_rise + WAIT_LOW_TIMEOUT_S
            while GPIO.input(self.ECHO) == 1 and monotonic_s() < t_fall_deadline:
                pass

            if GPIO.input(self.ECHO) == 1:
                detection = Detection(
                    side=self.side,
                    detection=False,
                    reason="pulse timeout",
                )
            else:
                t_fall = monotonic_s()
                pulse_s = t_fall - t_rise
                pulse_ms = pulse_s * 1000
                distance_m = (pulse_s * C) / 2
                distance_mm = distance_m * 1000

                echo_detected = 0 < pulse_s < self.THRESH_S

                detection = Detection(
                    side=self.side,
                    detection=True,
                    echo_detected=echo_detected,
                    pulse_ms=pulse_ms,
                    distance_mm=distance_mm,
                )

        if log:
            logger.info(detection.as_str())

        # Keep cycle rate sane
        elapsed = monotonic_s() - cycle_start
        if elapsed < CYCLE_MIN_S:
            time.sleep(CYCLE_MIN_S - elapsed)

        return True, detection



from bluer_objects.README.items import Items

from bluer_ugv.README.ravin.ravin3 import consts as ravin3
from bluer_ugv.README.ravin.ravin4 import consts as ravin4


items = Items(
    [
        {
            "name": "ravin3",
            "description": ravin3.description,
            "marquee": f"{ravin3.assets}/20250723_095155~2_1.gif",
            "url": "./ravin3",
        },
        {
            "name": "ravin4",
            "description": ravin4.description,
            "marquee": f"{ravin4.assets}/20251014_164022.jpg",
            "url": "./ravin4",
        },
    ]
)



import os

from bluer_options.help.functions import get_help
from bluer_objects import file, README

from bluer_ugv import NAME, VERSION, ICON, REPO_NAME
from bluer_ugv.help.functions import help_functions
from bluer_ugv.README import (
    alias,
    beast,
    eagle,
    fire,
    ravin,
    root,
    shield,
    arzhang,
    rangin,
    swallow,
    UGVs,
)


def build() -> bool:
    return all(
        README.build(
            items=readme.get("items", []),
            path=os.path.join(file.path(__file__), readme["path"]),
            cols=readme.get("cols", 3),
            ICON=ICON,
            NAME=NAME,
            VERSION=VERSION,
            REPO_NAME=REPO_NAME,
            help_function=lambda tokens: get_help(
                tokens,
                help_functions,
                mono=True,
            ),
            macros=readme.get("macros", {}),
        )
        for readme in root.docs
        + beast.docs
        + eagle.docs
        + fire.docs
        + ravin.docs
        + shield.docs
        + arzhang.docs
        + swallow.docs
        + alias.docs
        + UGVs.docs
        + rangin.docs
    )



from typing import Tuple


def generate_left_and_right(speed: int, steering: int) -> Tuple[int, int]:
    right = speed + steering
    left = speed - steering

    m = max(abs(left), abs(right), 100)
    left = left * 100 / m
    right = right * 100 / m

    return int(left), int(right)



from blueness import module
import numpy as np
import cv2
from typing import List, Dict

from bluer_options import string
from bluer_objects import file
from bluer_objects import objects
from bluer_objects.logger.stitch import stitch_images
from bluer_objects.graphics.gif import generate_animated_gif
from bluer_algo.socket.connection import SocketConnection
from bluer_algo.socket.message import SocketMessage

from bluer_ugv import NAME
from bluer_ugv.logger import logger


NAME = module.name(__file__, NAME)


def debug(
    object_name: str,
    generate_gif: bool = True,
    save_images: bool = True,
) -> bool:
    logger.info(
        "{}.debug -{}{}> {}".format(
            NAME,
            "images-" if save_images else "",
            "gif-" if generate_gif else "",
            object_name,
        )
    )

    socket = SocketConnection.listen_on()

    title = "debug..."

    cv2.namedWindow(title)
    logger.info("Ctrl+C to exit...")

    blank_image = np.zeros((480, 640, 3), np.uint8)

    image = blank_image.copy()
    dict_of_images: Dict[str, np.ndarray] = {}

    list_of_images: List[str] = []
    try:
        while True:
            cv2.imshow(title, np.flip(image, axis=2))
            cv2.waitKey(1)

            success, message = socket.receive_data(SocketMessage)
            if not success:
                break

            assert isinstance(message, SocketMessage)

            logger.info(f"message from {message.hostname}.")

            if "image" not in message.payload:
                logger.warning("no image.")
                continue

            dict_of_images[message.hostname] = message.payload["image"]

            image = stitch_images(
                [image for _, image in sorted(dict_of_images.items())]
            )

            if save_images:
                filename = objects.path_of(
                    filename="{}.png".format(string.timestamp()),
                    object_name=object_name,
                )

                if not file.save_image(filename, image, log=True):
                    break

                list_of_images.append(filename)
    except KeyboardInterrupt:
        logger.info("Ctrl+C, stopping.")

    cv2.destroyWindow(title)

    if generate_gif:
        if not generate_animated_gif(
            list_of_images,
            objects.path_of(
                filename=f"{object_name}.gif",
                object_name=object_name,
            ),
        ):
            return False

    return True



from bluer_objects.README.items import ImageItems

from bluer_ugv.README.swallow.consts import swallow_mechanical_designs

docs = [
    {
        "path": "../docs/swallow/digital/design/mechanical",
        "items": ImageItems(
            {
                f"{swallow_mechanical_designs}/robot.png": f"{swallow_mechanical_designs}/robot.stl",
                f"{swallow_mechanical_designs}/cage.png": f"{swallow_mechanical_designs}/cage.stl",
                f"{swallow_mechanical_designs}/measurements.png": "",
            }
        ),
    },
    {
        "path": "../docs/swallow/digital/design/mechanical/v1.md",
        "items": ImageItems(
            {
                f"{swallow_mechanical_designs}/v1/robot.png": f"{swallow_mechanical_designs}/v1/robot.stl",
                f"{swallow_mechanical_designs}/v1/cage.png": f"{swallow_mechanical_designs}/v1/cage.stl",
                f"{swallow_mechanical_designs}/v1/measurements.png": "",
            }
        ),
    },
]
